from .base import Model

__all__ = [
    "MatchMaker",
]


class MatchMaker(Model):
    """An implementation of the MatchMaker model.

    .. seealso:: https://github.com/AstraZeneca/chemicalx/issues/23
    """
