from .base import Model

__all__ = [
    "DeepDDS",
]


class DeepDDS(Model):
    """An implementation of the DeepDDS model.

    .. seealso:: https://github.com/AstraZeneca/chemicalx/issues/19
    """
