from .base import Model

__all__ = [
    "DeepCCI",
]


class DeepCCI(Model):
    """An implementation of the DeepCCI model.

    .. seealso:: https://github.com/AstraZeneca/chemicalx/issues/1
    """
