from .base import Model

__all__ = [
    "DeepDrug",
]


class DeepDrug(Model):
    """An implementation of the DeepDrug model.

    .. seealso:: https://github.com/AstraZeneca/chemicalx/issues/14
    """
