from .base import Model

__all__ = [
    "SSIDDI",
]


class SSIDDI(Model):
    """An implementation of the SSI-DDI model.

    .. seealso:: https://github.com/AstraZeneca/chemicalx/issues/11
    """
