from .base import Model

__all__ = [
    "EPGCNDS",
]


class EPGCNDS(Model):
    """An implementation of the EPGCNDS model.

    .. seealso:: https://github.com/AstraZeneca/chemicalx/issues/22
    """
