from .base import Model

__all__ = [
    "DeepDDI",
]


class DeepDDI(Model):
    """An implementation of the DeepDDI model.

    .. seealso:: https://github.com/AstraZeneca/chemicalx/issues/2
    """
