from .base import Model

__all__ = [
    "DeepSynergy",
]


class DeepSynergy(Model):
    """An implementation of the DeepSynergy model.

    .. seealso:: https://github.com/AstraZeneca/chemicalx/issues/16
    """
