from .base import Model

__all__ = [
    "MHCADDI",
]


class MHCADDI(Model):
    """An implementation of the MHCADDI model.

    .. seealso:: https://github.com/AstraZeneca/chemicalx/issues/13
    """
